#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example 5: CRUD (Create-Read-Update-Delete) example for Partitions.
"""

import sys
import logging
import yaml
import requests.packages.urllib3

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

example5 = examples.get("example5", None)
if example5 is None:
    print("example5 not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = example5.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = example5["hmc"]
cpcname = example5["cpcname"]
partname = example5["partname"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred["userid"]
password = cred["password"]

print(__doc__)

print("Using HMC %s with userid %s ..." % (hmc, userid))
session = zhmcclient.Session(hmc, userid, password)
cl = zhmcclient.Client(session)

timestats = example5.get("timestats", False)
if timestats:
    session.time_stats_keeper.enable()

print("Listing CPCs ...")
cpcs = cl.cpcs.list()
for cpc in cpcs:
    print(cpc.properties["name"], cpc.properties["status"],
          cpc.properties["object-uri"])

print("Finding CPC by name=%s ..." % cpcname)
try:
    cpc = cl.cpcs.find(name=cpcname)
except zhmcclient.NotFound:
    print("Could not find CPC %s on HMC %s" % (cpcname, hmc))
    sys.exit(1)

print("Checking if DPM is enabled on CPC %s..." % cpcname)
if cpc.dpm_enabled:
    print("CPC %s is in DPM mode." % cpcname)
    if cpc.properties['status'] != "active":
        print("CPC %s is not in 'active' state." % cpcname)
        sys.exit(1)
    try:
        print("Finding Partition by name=%s ..." % partname)
        partition = cpc.partitions.find(name=partname)
        if partition.properties['status'] == 'active':
            print("Stopping Partition %s ..." % partname)
            partition.stop()
        print("Deleting Partition %s ..." % partname)
        partition.delete()
    except zhmcclient.NotFound:
        print("Could not find Partition %s on CPC %s" % (partname, cpcname))

    print("Creating Partition %s on CPC %s ..." % (partname, cpcname))
    properties = dict()
    properties["name"] = partname
    properties["description"] = "Original partition description."
    properties["cp-processors"] = 2
    properties["initial-memory"] = 1024
    properties["maximum-memory"] = 2048
    properties["processor-mode"] = "shared"
    properties["boot-device"]  = "test-operating-system"
    print("Partition properties used:")
    print(properties)
    new_partition = cpc.partitions.create(properties)
    print("object-uri of new created Partition: %s" % new_partition)
    try:
        print("Finding Partition by name=%s ..." % partname)
        partition = cpc.partitions.find(name=partname)
        print("Starting Partition %s ..." % partname)
        partition.start()
        print("Pull full properties of Partition %s ..." % partname)
        partition.pull_full_properties()
        print("Description of Partition %s: %s" % (partname, partition.properties["description"]))
        print("Updating Partition %s properties ..." % partname)
        updated_properties = dict()
        updated_properties["description"] = "Updated partition description."
        partition.update_properties(updated_properties)
        print("Pull full properties of Partition %s ..." % partname)
        partition.pull_full_properties()
        print("Updated description of Partition %s: %s" % (partname, partition.properties["description"]))
    except zhmcclient.NotFound:
        print("Could not find Partition %s on CPC %s" % (partname, cpcname))
        sys.exit(1)

print("Logging off ...")
session.logoff()

if timestats:
    print(session.time_stats_keeper)

print("Done.")



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
{short description of this unit test}
"""

from __future__ import absolute_import

import re
import unittest
from mock import Mock

from zhmcclient import ...

class Test...(unittest.TestCase):

    def setUp(self):
        self..... # set up test object

    def test_{case1}(self):
        ... # execute the code to be tested
        ... # assert expected results using assert statement

    ... # more test functions

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _exceptions module.
"""

from __future__ import absolute_import

import unittest

from zhmcclient._exceptions import Error, ConnectionError, AuthError,\
    ParseError, VersionError, HTTPError

# TODO: Add tests for NoUniqueMatch
# TODO: Add tests for NotFound


class MyError(Error):
    """
    Concrete class to get instances of abstract base class ``Error``.
    """

    def __init__(self, *args, **kwargs):
        super(MyError, self).__init__(*args, **kwargs)


class TestError(unittest.TestCase):
    """
    Test the ``Error`` exception class.

    Because this is an abstract base class, we use our own derived class.
    """

    def test_empty(self):
        """Test Error exception with no argument."""

        exc = MyError()

        self.assertTrue(isinstance(exc, Error))
        self.assertTrue(isinstance(exc.args, tuple))
        self.assertEqual(len(exc.args), 0)

    def test_one(self):
        """Test Error exception with one argument."""

        exc = MyError('zaphod')

        self.assertEqual(len(exc.args), 1)
        self.assertEqual(exc.args[0], 'zaphod')

    def test_two(self):
        """Test with two arguments."""

        exc = MyError('zaphod', 42)

        self.assertEqual(len(exc.args), 2)
        self.assertEqual(exc.args[0], 'zaphod')
        self.assertEqual(exc.args[1], 42)

    def test_one_tuple(self):
        """Test Error exception with one argument that is a tuple of two
        items."""

        exc = MyError(('zaphod', 42))

        self.assertEqual(len(exc.args), 1)
        self.assertEqual(exc.args[0], ('zaphod', 42))


class SimpleTestMixin(object):
    """
    Mixin to test a number of simple exception classes.

    Simple exception classes take a message string as the single input
    argument.
    """

    exc_class = Exception  # Exception class to be tested

    def test_empty(self):
        """Test simple exception with no argument."""

        try:
            self.exc_class()
        except TypeError as e:
            self.assertRegexpMatches(
                e.args[0],
                r"__init__\(\) (takes.* 2 .*arguments .*1 .*given.*|"
                r"missing 1 .*argument.*)")
        except Exception as e:  # pylint: disable=broad-except
            self.fail("Exception was raised: %r" % e)
        else:
            self.fail("No exception was raised.")

    def test_one(self):
        """Test simple exception with one argument."""

        exc = self.exc_class('zaphod')

        self.assertTrue(isinstance(exc, Error))
        self.assertTrue(isinstance(exc.args, tuple))
        self.assertEqual(len(exc.args), 1)
        self.assertEqual(exc.args[0], 'zaphod')

    def test_two(self):
        """Test simple exception with two arguments."""

        try:
            self.exc_class('zaphod', 42)
        except TypeError as e:
            self.assertRegexpMatches(
                e.args[0],
                r"__init__\(\) takes.* 2 .*arguments .*3 .*given.*")
        except Exception as e:  # pylint: disable=broad-except
            self.fail("Exception was raised: %r" % e)
        else:
            self.fail("No exception was raised.")


class TestConnectionError(unittest.TestCase, SimpleTestMixin):
    """
    Test the simple exception class ``ConnectionError``.
    """

    def setUp(self):
        self.exc_class = ConnectionError


class TestAuthError(unittest.TestCase, SimpleTestMixin):
    """
    Test the simple exception class ``AuthError``.
    """

    def setUp(self):
        self.exc_class = AuthError


class TestParseError(unittest.TestCase, SimpleTestMixin):
    """
    Test the simple exception class ``ParseError``.
    """

    def setUp(self):
        self.exc_class = ParseError


class TestVersionError(unittest.TestCase, SimpleTestMixin):
    """
    Test the simple exception class ``VersionError``.
    """

    def setUp(self):
        self.exc_class = VersionError


class TestHTTPError(unittest.TestCase, SimpleTestMixin):
    """
    Test exception class ``HTTPError``.
    """

    def test_empty(self):
        """Test HTTPError with no arguments (expecting failure)."""

        try:
            # pylint: disable=no-value-for-parameter
            HTTPError()
        except TypeError as e:
            self.assertRegexpMatches(
                e.args[0],
                r"__init__\(\) (takes.* 2 .*arguments .*1 .*given.*|"
                r"missing 1 .*argument.*)")
        except Exception as e:  # pylint: disable=broad-except
            self.fail("Exception was raised: %r" % e)
        else:
            self.fail("No exception was raised.")

    def test_two(self):
        """Test HTTPError with two arguments (expecting failure)."""

        try:
            # pylint: disable=too-many-function-args
            HTTPError(dict(reason=42), 42)
        except TypeError as e:
            self.assertRegexpMatches(
                e.args[0],
                r"__init__\(\) takes.* 2 .*arguments .*3 .*given.*")
        except Exception as e:  # pylint: disable=broad-except
            self.fail("Exception was raised: %r" % e)
        else:
            self.fail("No exception was raised.")

    def test_one(self):
        """Test HTTPError with one argument."""

        resp_body = {
            'http-status': 404,
            'reason': 42,
            'message': 'abc def',
            'request-method': 'POST',
            'request-uri': '/api/cpcs/cpc1',
            'request-query-parms': [],
            'request-headers': {
                'content-type': 'application/json',
            },
            'request-authenticated-as': None,  # TODO: Real value
            'request-body': None,  # TODO: Real value
            'request-body-as-string': None,  # TODO: Real value
            'request-body-as-string-partial': None,  # TODO: Real value
            'stack': None,  # TODO: Real value
            'error-details': None,  # TODO: Real value
        }

        exc = HTTPError(resp_body)

        self.assertTrue(isinstance(exc, Error))
        self.assertEqual(len(exc.args), 1)
        self.assertTrue(isinstance(exc.args[0], dict))

        # Check the result properties:
        self.assertEqual(exc.http_status, resp_body['http-status'])
        self.assertEqual(exc.reason, resp_body['reason'])
        self.assertEqual(exc.message, resp_body['message'])
        self.assertEqual(exc.request_method, resp_body['request-method'])
        self.assertEqual(exc.request_uri, resp_body['request-uri'])
        self.assertEqual(exc.request_query_parms,
                         resp_body['request-query-parms'])
        self.assertEqual(exc.request_headers, resp_body['request-headers'])
        self.assertEqual(exc.request_authenticated_as,
                         resp_body['request-authenticated-as'])
        self.assertEqual(exc.request_body, resp_body['request-body'])
        self.assertEqual(exc.request_body_as_string,
                         resp_body['request-body-as-string'])
        self.assertEqual(exc.request_body_as_string_partial,
                         resp_body['request-body-as-string-partial'])
        self.assertEqual(exc.stack, resp_body['stack'])
        self.assertEqual(exc.error_details, resp_body['error-details'])

        # Check str()
        exp_str = str(resp_body['http-status']) + ',' + \
            str(resp_body['reason']) + ': ' + resp_body['message']
        self.assertEqual(str(exc), exp_str)

        # Check repr()
        exp_repr = 'HTTPError(' + str(resp_body['http-status']) + ', ' + \
            str(resp_body['reason']) + ', ' + resp_body['message'] + ', ...)'
        self.assertEqual(repr(exc), exp_repr)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A **Central Processor Complex (CPC)** is a physical z Systems or LinuxONE
computer. A particular HMC can manage multiple CPCs.

The HMC can manage a range of old and new CPC generations. Some older CPC
generations are not capable of supporting the HMC Web Services API; these older
CPCs can be managed using the GUI of the HMC, but not through its Web Services
API. Therefore, such older CPCs will not show up at the HMC Web Services API,
and thus will not show up in the API of this Python package.

TODO: List earliest CPC generation that supports the HMC Web Services API.

A CPC can be in any of the following three modes:

- DPM mode: Dynamic Partition Manager is enabled for the CPC.
- Ensemble mode: The CPC is member of an ensemble. This Python client
  does not support the functionality that is specific to ensemble mode.
- Classic mode: The CPC does not have Dynamic Partition Manager enabled,
  and is not member of an ensemble.

The functionality supported at the HMC API and thus also for users of this
Python client, depends on the mode in which the CPC currently is. If a
particular functionality is available only in a specific mode, that is
indicated in the description of the functionality.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._lpar import LparManager
from ._partition import PartitionManager
from ._activation_profile import ActivationProfileManager
from ._logging import _log_call


__all__ = ['CpcManager', 'Cpc']


class CpcManager(BaseManager):
    """
    Manager object for CPCs. This manager object is scoped to the HMC Web
    Services API capable CPCs managed by the HMC that is associated with a
    particular client.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.
    """

    def __init__(self, client):
        """
        Parameters:

          client (:class:`~zhmcclient.Client`):
            Client object for the HMC to be used.
        """
        super(CpcManager, self).__init__()
        self._session = client.session

    @_log_call
    def list(self, full_properties=False):
        """
        List the CPCs in scope of this manager object.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Cpc` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpcs_res = self.session.get('/api/cpcs')
        cpc_list = []
        if cpcs_res:
            cpc_items = cpcs_res['cpcs']
            for cpc_props in cpc_items:
                cpc = Cpc(self, cpc_props['object-uri'], cpc_props)
                if full_properties:
                    cpc.pull_full_properties()
                cpc_list.append(cpc)
        return cpc_list


class Cpc(BaseResource):
    """
    Representation of a CPC.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.
    """

    def __init__(self, manager, uri, properties):
        """
        Parameters:

          manager (:class:`~zhmcclient.CpcManager`):
            Manager object for this CPC.

          uri (string):
            Canonical URI path of the CPC object.

          properties (dict):
            Properties to be set for this resource object.
            See initialization of :class:`~zhmcclient.BaseResource` for
            details.
        """
        assert isinstance(manager, CpcManager)
        super(Cpc, self).__init__(manager, uri, properties)
        # We do here some lazy loading.
        self._lpars = None
        self._partitions = None
        self._reset_activation_profiles = None
        self._image_activation_profiles = None
        self._load_activation_profiles = None

    @property
    @_log_call
    def lpars(self):
        """
        :class:`~zhmcclient.LparManager`: Manager object for the LPARs in this
        CPC. `None`, if the CPC is in DPM mode.
        """
        # We do here some lazy loading.
        if not self._lpars:
            if self.dpm_enabled:
                self._lpars = None
            else:
                self._lpars = LparManager(self)
        return self._lpars

    @property
    @_log_call
    def partitions(self):
        """
        :class:`~zhmcclient.PartitionManager`: Manager object for the
        partitions in this CPC. `None`, if the CPC is not in DPM mode.
        """
        # We do here some lazy loading.
        if not self._partitions:
            if self.dpm_enabled:
                self._partitions = PartitionManager(self)
            else:
                self._partitions = None
        return self._partitions

    @property
    @_log_call
    def reset_activation_profiles(self):
        """
        :class:`~zhmcclient.ActivationManager`: Manager object for the
        Reset Activation Profiles in this CPC.
        `None`, if the CPC is in DPM mode.
        """
        # We do here some lazy loading.
        if not self._reset_activation_profiles:
            if self.dpm_enabled:
                self._reset_activation_profiles = None
            else:
                self._reset_activation_profiles = \
                    ActivationProfileManager(self, profile_type='reset')
        return self._reset_activation_profiles

    @property
    @_log_call
    def image_activation_profiles(self):
        """
        :class:`~zhmcclient.ActivationManager`: Manager object for the
        Image Activation Profiles in this CPC.
        `None`, if the CPC is in DPM mode.
        """
        # We do here some lazy loading.
        if not self._image_activation_profiles:
            if self.dpm_enabled:
                self._image_activation_profiles = None
            else:
                self._image_activation_profiles = \
                    ActivationProfileManager(self, profile_type='image')
        return self._image_activation_profiles

    @property
    @_log_call
    def load_activation_profiles(self):
        """
        :class:`~zhmcclient.ActivationManager`: Manager object for the
        Load Activation Profiles in this CPC.
        `None`, if the CPC is in DPM mode.
        """
        # We do here some lazy loading.
        if not self._load_activation_profiles:
            if self.dpm_enabled:
                self._load_activation_profiles = None
            else:
                self._load_activation_profiles = \
                    ActivationProfileManager(self, profile_type='load')
        return self._load_activation_profiles

    @property
    @_log_call
    def dpm_enabled(self):
        """
        bool: Indicates whether this CPC is currently in DPM mode
        (Dynamic Partition Manager mode).

        If the CPC is not currently in DPM mode, or if the CPC does not
        support DPM mode (i.e. before z13), `False` is returned.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        try:
            return self.get_property('dpm-enabled')
        except KeyError:
            return False

    def start(self, wait_for_completion=True):
        """
        Start this CPC, using the HMC operation "Start CPC".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            cpc_uri + '/operations/start',
            wait_for_completion=wait_for_completion)
        return result

    def stop(self, wait_for_completion=True):
        """
        Stop this CPC, using the HMC operation "Stop CPC".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:
          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            cpc_uri + '/operations/stop',
            wait_for_completion=wait_for_completion)
        return result

    @_log_call
    def import_profiles(self, profile_area, wait_for_completion=True):
        """
        Imports activation profiles and/or system activity profiles for the CPC
        from the SE hard drive into the CPC object using the HMC operation
        "Import Profiles".

        This operation is not permitted when the CPC is enabled for DPM.

        Parameters:

          profile_area (int):
             The numbered hard drive area (1-4) from which the profiles are
             imported.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.get_property('object-uri')
        body = {'profile-area': profile_area}
        result = self.manager.session.post(
            cpc_uri + '/operations/import-profiles', body,
            wait_for_completion=wait_for_completion)
        return result

    @_log_call
    def export_profiles(self, profile_area, wait_for_completion=True):
        """
        Exports activation profiles and/or system activity profiles from the
        CPC object designated to the SE hard drive using the HMC operation
        "Export Profiles".

        This operation is not permitted when the CPC is enabled for DPM.

        Parameters:

          profile_area (int):
             The numbered hard drive area (1-4) to which the profiles are
             exported. Any existing data is overwritten.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.get_property('object-uri')
        body = {'profile-area': profile_area}
        result = self.manager.session.post(
            cpc_uri + '/operations/export-profiles', body,
            wait_for_completion=wait_for_completion)
        return result



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _session module.
"""

from __future__ import absolute_import

import unittest
import requests
import requests_mock

from zhmcclient._session import Session


class SessionTests(unittest.TestCase):
    """
    Test the ``Session`` class.
    """

    # TODO: Test Session.get() in all variations (including errors)
    # TODO: Test Session.post() in all variations (including errors)
    # TODO: Test Session.delete() in all variations (including errors)

    @staticmethod
    def mock_server_1(m):
        """
        Set up the mocked responses for a simple HMC server that supports
        logon and logoff.
        """
        m.register_uri('POST', '/api/sessions',
                       json={'api-session': 'fake-session-id'},
                       headers={'X-Request-Id': 'fake-request-id'})
        m.register_uri('DELETE', '/api/sessions/this-session',
                       headers={'X-Request-Id': 'fake-request-id'},
                       status_code=204)

    def test_init(self):
        """Test initialization of Session object."""

        session = Session('fake-host', 'fake-user', 'fake-pw')

        self.assertEqual(session.host, 'fake-host')
        self.assertEqual(session.userid, 'fake-user')
        self.assertEqual(session._password, 'fake-pw')
        base_url = 'https://' + session.host + ':6794'
        self.assertEqual(session.base_url, base_url)
        self.assertTrue('Content-type' in session.headers)
        self.assertTrue('Accept' in session.headers)
        self.assertEqual(len(session.headers), 2)
        self.assertIsNone(session.session_id)
        self.assertTrue('X-API-Session' not in session.headers)
        self.assertIsNone(session.session)

    @requests_mock.mock()
    def test_logon_logoff(self, m):
        """Test logon and logoff; this uses post() and delete()."""

        self.mock_server_1(m)

        session = Session('fake-host', 'fake-user', 'fake-pw')

        self.assertIsNone(session.session_id)
        self.assertTrue('X-API-Session' not in session.headers)
        self.assertIsNone(session.session)

        logged_on = session.is_logon()

        self.assertFalse(logged_on)

        session.logon()

        self.assertEqual(session.session_id, 'fake-session-id')
        self.assertTrue('X-API-Session' in session.headers)
        self.assertTrue(isinstance(session.session, requests.Session))

        logged_on = session.is_logon()

        self.assertTrue(logged_on)

        session.logoff()

        self.assertIsNone(session.session_id)
        self.assertTrue('X-API-Session' not in session.headers)
        self.assertIsNone(session.session)

        logged_on = session.is_logon()

        self.assertFalse(logged_on)

    def test_delete_completed_job_status(self):
        """
        This tests the 'Delete Completed Job Status' operation.
        """
        session = Session('fake-host', 'fake-user', 'fake-id')
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'fake-session-id'})
            session.logon()
        job_uri = "/api/jobs/d9a3788e-683a-11e6-817c-00215e676926"
        with requests_mock.mock() as m:
            result = {}
            m.delete(job_uri, json=result)
            session.delete_completed_job_status(job_uri)

        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            session.logoff()

    def test_query_job_status(self):
        """
        This tests the 'Query Job Status' operation.
        """
        session = Session('fake-host', 'fake-user', 'fake-id')
        with requests_mock.mock() as m:
            # Because logon is deferred until needed, we perform it
            # explicitly in order to keep mocking in the actual test simple.
            m.post('/api/sessions', json={'api-session': 'fake-session-id'})
            session.logon()
        job_uri = "/api/jobs/d9a3788e-683a-11e6-817c-00215e676926"
        with requests_mock.mock() as m:
            result = {
                "job-reason-code": 0,
                "job-status-code": 204,
                "status": "complete"
            }
            m.get(job_uri, json=result)
            session.query_job_status(job_uri)

        with requests_mock.mock() as m:
            m.delete('/api/sessions/this-session', status_code=204)
            session.logoff()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Session class: A session to the HMC, optionally in context of an HMC user.
"""

from __future__ import absolute_import

import json
import time
import requests
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict

from ._exceptions import HTTPError, AuthError, ConnectionError
from ._timestats import TimeStatsKeeper
from ._logging import _get_logger, _log_call

LOG = _get_logger(__name__)

__all__ = ['Session']

_HMC_PORT = 6794
_HMC_SCHEME = "https"
_STD_HEADERS = {
    'Content-type': 'application/json',
    'Accept': '*/*'
}


class Session(object):
    """
    A session to the HMC, optionally in context of an HMC user.

    The session supports operations that require to be authenticated, as well
    as operations that don't (e.g. obtaining the API version).

    The session can keep statistics about the elapsed time for issuing HTTP
    requests against the HMC API. Instance variable
    :attr:`~zhmcclient.Session.time_stats_keeper` is used to enable/disable the
    measurements, and to print the statistics.
    """

    def __init__(self, host, userid=None, password=None):
        """
        Creating a session object will not immediately cause a logon to be
        attempted; the logon is deferred until needed.

        Parameters:

          host (:term:`string`):
            HMC host. For valid formats, see the
            :attr:`~zhmcclient.Session.host` property.
            Must not be `None`.

          userid (:term:`string`):
            Userid of the HMC user to be used.
            If `None`, only operations that do not require authentication, can
            be performed.

          password (:term:`string`):
            Password of the HMC user to be used.

        TODO: Add support for client-certificate-based authentication.
        """
        self._host = host
        self._userid = userid
        self._password = password
        self._base_url = "{scheme}://{host}:{port}".format(
            scheme=_HMC_SCHEME,
            host=self._host,
            port=_HMC_PORT)
        self._headers = _STD_HEADERS  # dict with standard HTTP headers
        self._session_id = None  # HMC session ID
        self._session = None  # requests.Session() object
        self._time_stats_keeper = TimeStatsKeeper()
        self._headers = _STD_HEADERS
        self._session_id = None
        LOG.debug("Created session object for '%(user)s' on '%(host)s'",
                  {'user': self._userid, 'host': self._host})

    @property
    def host(self):
        """
        :term:`string`: HMC host, in one of the following formats:

          * a short or fully qualified DNS hostname
          * a literal (= dotted) IPv4 address
          * a literal IPv6 address, formatted as defined in :term:`RFC3986`
            with the extensions for zone identifiers as defined in
            :term:`RFC6874`, supporting ``-`` (minus) for the delimiter
            before the zone ID string, as an additional choice to ``%25``
        """
        return self._host

    @property
    def userid(self):
        """
        :term:`string`: Userid of the HMC user to be used.

        If `None`, only operations that do not require authentication, can be
        performed.
        """
        return self._userid

    @property
    def base_url(self):
        """
        :term:`string`: Base URL of the HMC in this session.

        Example:

        ::

            https://myhmc.acme.com:6794
        """
        return self._base_url

    @property
    def headers(self):
        """
        :term:`header dict`: HTTP headers to be used in each request.

        Initially, this is the following set of headers:

        ::

            Content-type: application/json
            Accept: */*

        When the session is logged on to the HMC, the session token is added
        to these headers:

        ::

            X-API-Session: ...
        """
        return self._headers

    @property
    def time_stats_keeper(self):
        """
        The time statistics keeper (for a usage example, see section
        :ref:`Time Statistics`).
        """
        return self._time_stats_keeper

    @property
    def session_id(self):
        """
        :term:`string`: Session ID for this session, returned by the HMC.
        """
        return self._session_id

    @property
    def session(self):
        """
        :term:`string`: :class:`requests.Session` object for this session.
        """
        return self._session

    @_log_call
    def logon(self):
        """
        Make sure the session is logged on to the HMC.

        After successful logon to the HMC, the following is stored in this
        session object for reuse in subsequent operations:

        * the HMC session ID, in order to avoid extra userid authentications,
        * a :class:`requests.Session` object, in order to enable connection
          pooling. Connection pooling avoids repetitive SSL/TLS handshakes.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if not self.is_logon():
            self._do_logon()

    @_log_call
    def logoff(self):
        """
        Make sure the session is logged off from the HMC.

        After successful logoff, the HMC session ID and
        :class:`requests.Session` object stored in this object are reset.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if self.is_logon():
            self._do_logoff()

    @_log_call
    def is_logon(self):
        """
        Return a boolean indicating whether the session is currently logged on
        to the HMC.
        """
        return self._session_id is not None

    def _do_logon(self):
        """
        Log on, unconditionally. This can be used to re-logon.
        This requires credentials to be provided.
        """
        if self._userid is None or self._password is None:
            raise AuthError("Userid or password not provided.")
        logon_uri = '/api/sessions'
        logon_body = {
            'userid': self._userid,
            'password': self._password
        }
        self._headers.pop('X-API-Session', None)  # Just in case
        self._session = requests.Session()
        logon_res = self.post(logon_uri, logon_body, logon_required=False)
        self._session_id = logon_res['api-session']
        self._headers['X-API-Session'] = self._session_id

    def _do_logoff(self):
        """
        Log off, unconditionally.
        """
        session_uri = '/api/sessions/this-session'
        self.delete(session_uri, logon_required=False)
        self._session_id = None
        self._session = None
        self._headers.pop('X-API-Session', None)

    def _log_hmc_request_id(self, response):
        """
        Log the identifier the HMC uses to distinguish requests.
        """
        LOG.info("Returned HMC request ID: %r",
                 response.headers.get('X-Request-Id', ''))

    def _log_http_method(self, http_method, uri):
        """
        Log HTTP method name and target URI.
        """
        LOG.info("HTTP request: %s %s", http_method, uri)

    @_log_call
    def get(self, uri, logon_required=True):
        """
        Perform the HTTP GET method against the resource identified by a URI.

        A set of standard HTTP headers is automatically part of the request.

        If the HMC session token is expired, this method re-logs on and retries
        the operation.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g. "/api/session".
            This URI is relative to the base URL of the session (see
            the :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC. For example, the API version retrieval
            operation does not require that.

        Returns:

          :term:`json object` with the operation result.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if logon_required:
            self.logon()
        url = self.base_url + uri
        self._log_http_method('GET', uri)
        stats = self.time_stats_keeper.get_stats('get ' + uri)
        stats.begin()
        req = self._session or requests
        try:
            result = req.get(url, headers=self.headers, verify=False)
            self._log_hmc_request_id(result)
        except requests.exceptions.RequestException as exc:
            raise ConnectionError(str(exc))
        finally:
            stats.end()

        if result.status_code == 200:
            return result.json(object_pairs_hook=OrderedDict)
        elif result.status_code == 403:
            reason = result.json().get('reason', None)
            if reason == 5:
                # API session token expired: re-logon and retry
                if logon_required:
                    self._do_logon()
                else:
                    raise AuthError("API session token unexpectedly expired "
                                    "for GET on resource that does not "
                                    "require authentication: {}".
                                    format(uri))
                return self.get(uri, logon_required)
            else:
                exc = HTTPError(result.json(object_pairs_hook=OrderedDict))
                raise AuthError("HTTP authentication failed: {}".
                                format(str(exc)))
        else:
            raise HTTPError(result.json(object_pairs_hook=OrderedDict))

    @_log_call
    def post(self, uri, body=None, logon_required=True,
             wait_for_completion=True):
        """
        Perform the HTTP POST method against the resource identified by a URI,
        using a provided request body.

        A set of standard HTTP headers is automatically part of the request.

        HMC operations using HTTP POST are either synchronous or asynchronous.
        Asynchronous operations return the URI of an asynchronously executing
        job that can be queried for status and result.

        Examples for synchronous operations:

        * With no response body: "Logon", "Update CPC Properties"
        * With a response body: "Create Partition"

        Examples for asynchronous operations:

        * With no ``job-results`` field in the completed job status response:
          "Start Partition"
        * With a ``job-results`` field in the completed job status response
          (under certain conditions): "Activate a Blade", or "Set CPC Power
          Save"

        The `wait_for_completion` parameter of this method can be used to deal
        with asynchronous HMC operations in a synchronous way.

        If executing the operation reveals that the HMC session token is
        expired, this method re-logs on and retries the operation.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g. "/api/session".
            This URI is relative to the base URL of the session (see the
            :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          body (:term:`json object`):
            JSON object to be used as the HTTP request body (payload).
            `None` means the same as an empty dictionary, namely that no HTTP
            body is included in the request.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC. For example, the "Logon" operation does
            not require that.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested HMC operation, as follows:

            * If `True`, this method will wait for completion of the requested
              operation, regardless of whether the operation is synchronous or
              asynchronous.

            * If `False`, this method will immediately return the result of the
              HTTP POST method, regardless of whether the operation is
              synchronous or asynchronous.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns a JSON object
            representing the response body of the synchronous operation, or the
            response body of the completed job that performed the asynchronous
            operation. If a synchronous operation has no response body, `None`
            is returned.

            If `wait_for_completion` is `False`, returns a JSON object
            representing the response body of the synchronous or asynchronous
            operation. In case of an asynchronous operation, the JSON object
            will have a member named ``job-uri``, whose value can be used with
            the :meth:`~zhmcclient.Session.query_job_status` method to
            determine the status of the job and the result of the original
            operation, once the job has completed.

            See the section in the :term:`HMC API` about the specific HMC
            operation and about the "Query Job Status" operation, for a
            description of the members of the returned JSON objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if logon_required:
            self.logon()
        url = self.base_url + uri
        self._log_http_method('POST', uri)
        stats = self.time_stats_keeper.get_stats('post ' + uri)
        stats.begin()
        req = self._session or requests
        try:
            if body is None:
                result = req.post(url, headers=self.headers,
                                  verify=False)
            else:
                data = json.dumps(body)
                result = req.post(url, data=data, headers=self.headers,
                                  verify=False)
            self._log_hmc_request_id(result)
        except requests.exceptions.RequestException as exc:
            raise ConnectionError(str(exc))
        finally:
            stats.end()

        if result.status_code in (200, 201):
            return result.json(object_pairs_hook=OrderedDict)
        elif result.status_code == 204:
            # No content
            return None
        elif result.status_code == 202:
            job_uri = result.json()['job-uri']
            job_url = self.base_url + job_uri
            if not wait_for_completion:
                return result.json(object_pairs_hook=OrderedDict)
            while 1:
                self._log_http_method('GET', job_uri)
                stats = self.time_stats_keeper.get_stats('get ' + job_uri)
                stats.begin()
                try:
                    result = req.get(job_url, headers=self.headers,
                                     verify=False)
                    self._log_hmc_request_id(result)
                except requests.exceptions.RequestException as exc:
                    raise ConnectionError(str(exc))
                finally:
                    stats.end()
                if result.status_code in (200, 204):
                    if result.json()['status'] == 'complete':
                        self.delete_completed_job_status(job_uri)
                        return result.json(object_pairs_hook=OrderedDict)
                    else:
                        # TODO: Add support for timeout
                        time.sleep(1)  # Avoid hot spin loop
                else:
                    raise HTTPError(result.json(object_pairs_hook=OrderedDict))
        elif result.status_code == 403:
            reason = result.json().get('reason', None)
            if reason == 5:
                # API session token expired: re-logon and retry
                if logon_required:
                    self._do_logon()
                else:
                    raise AuthError("API session token unexpectedly expired "
                                    "for POST on resource that does not "
                                    "require authentication: {}".
                                    format(uri))
                return self.post(uri, body, logon_required)
            else:
                exc = HTTPError(result.json(object_pairs_hook=OrderedDict))
                raise AuthError("HTTP authentication failed: {}".
                                format(str(exc)))
        else:
            raise HTTPError(result.json(object_pairs_hook=OrderedDict))

    @_log_call
    def delete(self, uri, logon_required=True):
        """
        Perform the HTTP DELETE method against the resource identified by a
        URI.

        A set of standard HTTP headers is automatically part of the request.

        If the HMC session token is expired, this method re-logs on and retries
        the operation.

        Parameters:

          uri (:term:`string`):
            Relative URI path of the resource, e.g.
            "/api/session/{session-id}".
            This URI is relative to the base URL of the session (see
            the :attr:`~zhmcclient.Session.base_url` property).
            Must not be `None`.

          logon_required (bool):
            Boolean indicating whether the operation requires that the session
            is logged on to the HMC. For example, for the logoff operation, it
            does not make sense to first log on.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        if logon_required:
            self.logon()
        url = self.base_url + uri
        self._log_http_method('DELETE', uri)
        stats = self.time_stats_keeper.get_stats('delete ' + uri)
        stats.begin()
        req = self._session or requests
        try:
            result = req.delete(url, headers=self.headers, verify=False)
            self._log_hmc_request_id(result)
        except requests.exceptions.RequestException as exc:
            raise ConnectionError(str(exc))
        finally:
            stats.end()

        if result.status_code in (200, 204):
            return
        elif result.status_code == 403:
            reason = result.json().get('reason', None)
            if reason == 5:
                # API session token expired: re-logon and retry
                if logon_required:
                    self._do_logon()
                else:
                    raise AuthError("API session token unexpectedly expired "
                                    "for DELETE on resource that does not "
                                    "require authentication: {}".
                                    format(uri))
                self.delete(uri, logon_required)
                return
            else:
                exc = HTTPError(result.json(object_pairs_hook=OrderedDict))
                raise AuthError("HTTP authentication failed: {}".
                                format(str(exc)))
        else:
            raise HTTPError(result.json(object_pairs_hook=OrderedDict))

    @_log_call
    def query_job_status(self, job_uri):
        """
        Perform the "Query Job Status" operation on a job identified by its
        URI and return the status of the job. If the job is complete, the
        return value also contains the result of the operation the job was
        performing asynchronously.

        A set of standard HTTP headers is automatically part of the request.

        If the HMC session token is expired, this method re-logs on and retries
        the operation.

        Parameters:

          job_uri (:term:`string`):
            Job URI; e.g. from the value of the ``job-uri`` field of the
            result of the original operation that was performed asynchronously
            by the job.
            Must not be `None`.

        Returns:

          :term:`json object`:

            A JSON object indicating the status of the job in its ``status``
            member, and if the job is complete (``status='complete'``), also
            with members ``job-status-code``, ``job-reason-code``, and
            optionally ``job-results``.

            For details, see the sections in the :term:`HMC API` about the
            "Query Job Status" operation and about the original operation that
            was performed asynchronously.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        result = self.get(job_uri)
        return result

    @_log_call
    def delete_completed_job_status(self, job_uri):
        """
        Perform the "Delete completed Job Status" operation on a job identified
        by its URI and return the status of the job.

        A set of standard HTTP headers is automatically part of the request.

        If the HMC session token is expired, this method re-logs on and retries
        the operation.

        Parameters:

          job_uri (:term:`string`):
            Job URI; e.g. from the value of the ``job-uri`` field of the
            result of the original operation that was performed asynchronously
            by the job.
            Must not be `None`.

        Returns:

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        self.delete(job_uri)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Base definitions for resource classes.

Resource objects represent the real manageable resources in the systems managed
by the HMC.
"""

from __future__ import absolute_import
import time

__all__ = ['BaseResource']


class BaseResource(object):
    """
    Abstract base class for resource classes (e.g. :class:`~zhmcclient.Cpc`)
    representing manageable resources.

    Such resource objects are representations of real manageable resources in
    the systems managed by the HMC.

    It defines the interface for the derived resource classes, and implements
    methods that have a common implementation for the derived resource classes.
    """

    def __init__(self, manager, uri, properties):
        """
        Parameters:

          manager (subclass of :class:`~zhmcclient.BaseManager`):
            Manager object for this resource (and for all resources of the same
            type in the scope of that manager).

          uri (string):
            Canonical URI path of the BaseResource object.

          properties (dict):
            Properties to be set for this resource object.

            * Key: Name of the property.
            * Value: Value of the property.

            The input dictionary is copied (shallow), so that the input
            dictionary can be modified by the user without affecting the
            properties of resource objects created from that input dictionary.

            Property qualifiers (read-only, etc.) are not represented on the
            resource object. The properties on the resource object are
            mutable. Whether or not a particular property of the represented
            manageable resource can be updated is described in its property
            qualifiers. See section "Property characteristics" in chapter 5 of
            :term:`HMC API` for a description of the concept of property
            qualifiers, and the respective sections describing the resources
            for their actual definitions of property qualifiers.
        """
        self._manager = manager
        self._uri = uri
        self._properties = dict(properties)
        self._properties_timestamp = int(time.time())
        self._full_properties = False

    @property
    def properties(self):
        """
        dict:
          The properties of this resource object.

          * Key: Name of the property.
          * Value: Value of the property.

          See the respective sections in :term:`HMC API` for a description
          of the resources along with their properties.

          The dictionary contains either the full set of resource properties,
          or the short set of resource properties as obtained by list
          operations.
        """
        return self._properties

    @property
    def manager(self):
        """
        Subclass of :class:`~zhmcclient.BaseManager`:
          Manager object for this resource (and for all resources of the same
          type in the scope of that manager).
        """
        return self._manager

    @property
    def full_properties(self):
        """
        A boolean indicating whether the resource properties in this object
        are the full set of resource properties, vs. just the short set of
        resource properties as obtained by list functions.
        """
        return self._full_properties

    @property
    def properties_timestamp(self):
        """
        The point in time of the last update of the resource properties
        in this object, as Unix time (an integer that is the number of seconds
        since the Unix epoch).
        """
        return self._properties_timestamp

    def pull_full_properties(self):
        """
        Retrieve the full set of properties for this resource (and store them
        in :attr:`properties`).

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        full_properties = self.manager.session.get(self._uri)
        self._properties = dict(full_properties)
        self._properties_timestamp = int(time.time())
        self._full_properties = True

    def get_property(self, name):
        """
        Return the value of a resource property. If the resource property
        is not in the currently known set of properties, the full set of
        resource properties is retrieved and the resource property is
        again attempted to be returned.

        Returns:

          The value of the resource property.

        Raises:

          KeyError: The resource property could not be found (also not in the
            full set of resource properties).
          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        try:
            return self._properties[name]
        except KeyError:
            if self.full_properties:
                raise
            self.pull_full_properties()
            return self._properties[name]



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A **partition** is a subset of a physical z Systems or LinuxONE computer
that is in DPM mode (Dynamic Partition Manager mode).
Objects of this class are not provided when the CPC is not in DPM mode.

A partition is always contained in a CPC.

Partitions can be created and deleted dynamically, and their resources such
as CPU, memory or I/O devices can be configured dynamically.
You can create as many partition definitions as you want, but only a specific
number of partitions can be active at any given time.

TODO: How can a user find out what the maximum is, before it is reached?
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource

__all__ = ['PartitionManager', 'Partition']


class PartitionManager(BaseManager):
    """
    Manager object for Partitions. This manager object is scoped to the
    partitions of a particular CPC.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.
    """

    def __init__(self, cpc):
        """
        Parameters:

          cpc (:class:`~zhmcclient.Cpc`):
            CPC defining the scope for this manager object.
        """
        super(PartitionManager, self).__init__(cpc)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: Parent object (CPC) defining the scope for
        this manager object.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        List the partitions in scope of this manager object.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Partition` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        partitions_res = self.session.get(cpc_uri + '/partitions')
        partition_list = []
        if partitions_res:
            partition_items = partitions_res['partitions']
            for partition_props in partition_items:
                partition = Partition(self, partition_props['object-uri'],
                                      partition_props)
                if full_properties:
                    partition.pull_full_properties()
                partition_list.append(partition)
        return partition_list

    def create(self, properties):
        """
        Create a partition with the specified resource properties.

        Parameters:

          properties (dict): Properties for the new partition.

        Returns:

          string: The resource URI of the new partition.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        result = self.session.post(cpc_uri + '/partitions', body=properties)
        return result['object-uri']


class Partition(BaseResource):
    """
    Representation of a Partition.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.
    """

    def __init__(self, manager, uri, properties):
        """
        Parameters:

          manager (:class:`~zhmcclient.PartitionManager`):
            Manager object for this resource.

          uri (string):
            Canonical URI path of the Partition object.

          properties (dict):
            Properties to be set for this resource object.
            See initialization of :class:`~zhmcclient.BaseResource` for
            details.
        """
        assert isinstance(manager, PartitionManager)
        super(Partition, self).__init__(manager, uri, properties)

    def start(self, wait_for_completion=True):
        """
        Start (activate) this partition, using the HMC operation "Start
        Partition".

        TODO: Describe what happens if the maximum number of active partitions
        is exceeded.

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/start',
            wait_for_completion=wait_for_completion)
        return result

    def stop(self, wait_for_completion=True):
        """
        Stop (deactivate) this partition, using the HMC operation "Stop
        Partition".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        result = self.manager.session.post(
            partition_uri + '/operations/stop',
            wait_for_completion=wait_for_completion)
        return result

    def delete(self):
        """
        Deletes this partition.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.delete(partition_uri)

    def update_properties(self, properties):
        """
        Updates one or more of the writable properties of a partition
        with the specified resource properties.

        Parameters:

          properties (dict): Updated properties for the partition.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        partition_uri = self.get_property('object-uri')
        self.manager.session.post(partition_uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
**Activation profiles** are required for CPC (processor)
and CPC image (partition) activation. They are used to tailor the operation
of a CPC and are stored in the Support Element associated with the CPC.

There are types of activation profiles:

1. Reset:
   Every CPC in the processor cluster requires a reset profile to determine
   the mode in which the CPC licensed internal code will be loaded and
   how much central storage and expanded storage will be used.

2. Image:
   If LPAR mode is selected in the reset profile, each partition
   can have an image profile. The image profile determines the number of CPs
   that the image will use and whether these CPs will be dedicated
   to the partition or shared. It also allows you to assign the amount of
   central storage and expanded storage that will be used by each partition.

3. Load:
   A load profile is needed to define the channel address of the device that
   the operating system will be loaded from.

Activation Profiles are not provided when the CPC is enabled for DPM.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._logging import _log_call


__all__ = ['ActivationProfileManager', 'ActivationProfile']


class ActivationProfileManager(BaseManager):
    """
    Manager object for Activation Profiles.
    This manager object is scoped to the Activation Profiles of a particular
    CPC.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.
    """

    def __init__(self, cpc, profile_type):
        """
        Parameters:

          cpc (:class:`~zhmcclient.Cpc`):
            CPC defining the scope for this manager object.

          profile_type (string):
            Controls which type of Activation Profiles
            this manager returns.

            * If `reset`, this manager returns Reset Activation Profiles.
            * If `image`, this manager returns Image Activation Profiles.
            * If `load`, this manager returns Load Activation Profiles.
        """
        super(ActivationProfileManager, self).__init__(cpc)
        self._profile_type = profile_type

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: Parent object (CPC) defining the scope for
        this manager object.
        """
        return self._parent

    @property
    def profile_type(self):
        """
        Returns type of Activation Profiles:
          * If `reset`, this manager returns Reset Activation Profiles.
          * If `image`, this manager returns Image Activation Profiles.
          * If `load`, this manager returns Load Activation Profiles.
        """
        return self._profile_type

    @_log_call
    def list(self, full_properties=False):
        """
        List the Activation Profiles in scope of this manager object.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.ActivationProfile` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        activation_profile = self._profile_type + '-activation-profiles'
        profiles_res = self.session.get(cpc_uri + '/' + activation_profile)
        profile_list = []
        if profiles_res:
            profile_items = profiles_res[self._profile_type +
                                         '-activation-profiles']
            for profile_props in profile_items:
                profile = ActivationProfile(self, profile_props['element-uri'],
                                            profile_props)
                if full_properties:
                    profile.pull_full_properties()
                profile_list.append(profile)
        return profile_list


class ActivationProfile(BaseResource):
    """
    Representation of an Activation Profile.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.
    """

    def __init__(self, manager, uri, properties):
        """
        Parameters:

          manager (:class:`~zhmcclient.ActivationProfileManager`):
            Manager object for this resource.

          uri (string):
            Canonical URI path of the Activation Profile object.

          properties (dict):
            Properties to be set for this resource object.
            See initialization of :class:`~zhmcclient.BaseResource` for
            details.
        """
        assert isinstance(manager, ActivationProfileManager)
        super(ActivationProfile, self).__init__(manager, uri, properties)

    def update_properties(self, properties):
        """
        Updates one or more of the writable properties of
        an activation profile with the specified resource properties.

        Parameters:

          properties (dict): Updated properties for the activation profile.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        profile_uri = self.get_property('element-uri')
        self.manager.session.post(profile_uri, body=properties)



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Base definitions for resource manager classes.

Resource manager classes exist for each resource type and are helper classes
that provide functionality common for the resource type.

Resource manager objects are not necessarily singleton objects, because they
have a scope of a certain set of resource objects. For example, the resource
manager object for LPARs exists once for each CPC managed by the HMC, and the
resource object scope of each LPAR manager object is the set of LPARs in that
CPC.
"""

from __future__ import absolute_import

from ._exceptions import NotFound, NoUniqueMatch

__all__ = ['BaseManager']


class BaseManager(object):
    """
    Abstract base class for manager classes (e.g.
    :class:`~zhmcclient.CpcManager`).

    It defines the interface for the derived manager classes, and implements
    methods that have a common implementation for the derived manager classes.
    """

    def __init__(self, parent=None):
        """
        Parameters:

          parent (subclass of :class:`~zhmcclient.BaseResource`):
            Parent resource defining the scope for this manager.

            `None`, if the manager has no parent, i.e. when it manages
            top-level resources.
        """
        self._parent = parent
        self._session = parent.manager.session if parent else None
        # Note: Managers of top-level resources must set session in their init.

    @property
    def session(self):
        """
        :class:`~zhmcclient.Session`:
          Session with the HMC.
        """
        assert self._session is not None
        return self._session

    @property
    def parent(self):
        """
        Subclass of :class:`~zhmcclient.BaseResource`:
          Parent resource defining the scope for this manager.

          `None`, if the manager has no parent, i.e. when it manages top-level
          resources.
        """
        return self._parent

    def list(self, full_properties=False):
        """
        Interface for the list function that is used by the :meth:`find` and
        :meth:`findall` methods.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          List of resource objects managed by this manager.

        Raises:

          Exceptions raised by the `list()` method in the derived classes.
        """
        raise NotImplementedError

    def find(self, **kwargs):
        """
        Find exactly one resource that is managed by this manager, by the value
        of zero or more resource attributes.

        If more than one attribute is specified, all attributes need to match
        for the resource to be found.

        Keyword Arguments:

          : Each keyword argument is used to filter the resources managed by
            this manager, whereby the name of the keyword argument is used to
            look up the same-named resource attribute, and the value of the
            keyword argument is used to compare the resource's attribute value
            against.

        Returns:

          : The single resource object that was found.

        Raises:

          :exc:`~zhmcclient.NotFound`
          :exc:`~zhmcclient.NoUniqueMatch`
          Exceptions raised by :meth:`~zhmcclient.BaseManager.list`
        """
        matches = self.findall(**kwargs)
        num_matches = len(matches)
        if num_matches == 0:
            raise NotFound
        elif num_matches > 1:
            raise NoUniqueMatch
        else:
            return matches[0]

    def findall(self, **kwargs):
        """
        Find zero or more resources that are managed by this manager, by the
        value of zero or more resource properties.

        If more than one property is specified, all properties need to match
        for the resources to be found.

        Keyword Arguments:

          : Each keyword argument is used to filter the resources managed by
            this manager, whereby the name of the keyword argument is used to
            look up the same-named resource property, and the value of the
            keyword argument is used to compare the resource's property value
            against.

        Returns:

          : A list of zero or more resource objects that were found.

        Raises:

          Exceptions raised by :meth:`~zhmcclient.BaseManager.list`.
        """
        searches = kwargs.items()
        found = list()
        listing = self.list()
        for obj in listing:
            try:
                if all(obj.get_property(propname) == value
                       for (propname, value) in searches):
                    found.append(obj)
            except AttributeError:
                continue
        return found



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _resource module.
"""

from __future__ import absolute_import

import unittest
import time

from zhmcclient._resource import BaseResource
from zhmcclient._manager import BaseManager


class MyResource(BaseResource):
    """
    A derived resource for testing the (abstract) BaseResource class.
    """

    def __init__(self, *args, **kwargs):
        super(MyResource, self).__init__(*args, **kwargs)


class MyManager(BaseManager):
    """
    A derived resource manager for testing purposes.

    It is only needed because BaseResource needs it; it is not subject
    of test in this unit test module.
    """

    def __init__(self, *args, **kwargs):
        super(MyManager, self).__init__(*args, **kwargs)

    def list(self, full_properties=False):
        pass  # to avoid warning about unimplemented abstract method.


class ResourceTestCase(unittest.TestCase):
    """
    Base class for all tests in this file.
    """

    def setUp(self):
        self.mgr = MyManager()
        self.uri = "/api/resource/deadbeef-beef-beef-beef-deadbeefbeef"

    def assert_properties(self, resource, exp_props):
        """
        Assert that the properties of a resource object are as expected.
        """

        # Check that the properties member is a dict
        self.assertTrue(isinstance(resource.properties, dict))

        # Verify that the resource properties are as expected
        self.assertEqual(len(resource.properties), len(exp_props))
        for key, value in exp_props.items():
            self.assertEqual(resource.properties[key], value)


class InitTests(ResourceTestCase):
    """Test BaseResource initialization."""

    def test_empty(self):
        """Test with an empty set of input properties."""
        init_props = {}

        res = MyResource(self.mgr, self.uri, init_props)

        self.assertTrue(res.manager is self.mgr)
        self.assertEqual(res._uri, self.uri)
        self.assert_properties(res, init_props)
        self.assertTrue(int(time.time()) - res.properties_timestamp <= 1)
        self.assertEqual(res.full_properties, False)

    def test_simple(self):
        """Test with a simple set of input properties."""
        init_props = {
            'prop1': 'abc',
            'prop2': 100042
        }

        res = MyResource(self.mgr, self.uri, init_props)

        self.assertTrue(res.manager is self.mgr)
        self.assertEqual(res._uri, self.uri)
        self.assert_properties(res, init_props)
        self.assertTrue(int(time.time()) - res.properties_timestamp <= 1)
        self.assertEqual(res.full_properties, False)

    def test_prop_case(self):
        """Test case sensitivity for the input properties."""
        init_props = {
            'prop1': 'abc',
            'Prop1': 100042
        }

        res = MyResource(self.mgr, self.uri, init_props)

        self.assertTrue(res.manager is self.mgr)
        self.assertEqual(res._uri, self.uri)
        self.assert_properties(res, init_props)
        self.assertTrue(int(time.time()) - res.properties_timestamp <= 1)
        self.assertEqual(res.full_properties, False)

    def test_invalid_type(self):
        """Test that input properties with an invalid type fail."""
        init_props = 42
        try:

            MyResource(self.mgr, self.uri, init_props)

        except TypeError:
            pass
        else:
            self.fail("TypeError was not raised when initializing resource "
                      "with invalid properties: %r" % init_props)


class PropertySetTests(ResourceTestCase):
    """Test BaseResource by setting properties."""

    def test_add_to_empty(self):
        """Test setting a property in a resource object with no properties."""
        init_props = {}
        set_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        res_props = dict(init_props)
        res_props.update(set_props)
        res = MyResource(self.mgr, self.uri, init_props)

        for key, value in set_props.items():
            res.properties[key] = value

        self.assert_properties(res, res_props)

    def test_replace_one_add_one(self):
        """Test replacing and adding a property in a resource object."""
        init_props = {
            'prop1': 42,
        }
        set_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        res_props = dict(init_props)
        res_props.update(set_props)
        res = MyResource(self.mgr, self.uri, init_props)

        for key, value in set_props.items():
            res.properties[key] = value

        self.assert_properties(res, res_props)


class PropertyDelTests(ResourceTestCase):
    """Test BaseResource by deleting properties."""

    def test_del_one(self):
        """Test deleting a property in a resource object."""
        init_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        del_keys = ('prop1',)
        res_props = {
            'prop2': 100042
        }
        res = MyResource(self.mgr, self.uri, init_props)

        for key in del_keys:
            del res.properties[key]

        self.assert_properties(res, res_props)

    def test_del_all(self):
        """Test deleting all properties in a resource object."""
        init_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        del_keys = ('prop1', 'prop2')
        res_props = {}
        res = MyResource(self.mgr, self.uri, init_props)

        for key in del_keys:
            del res.properties[key]

        self.assert_properties(res, res_props)

    def test_del_invalid(self):
        """Test deleting an invalid property in a resource object."""
        init_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        org_init_props = dict(init_props)
        res = MyResource(self.mgr, self.uri, init_props)
        invalid_key = 'inv1'
        try:

            del res.properties[invalid_key]

        except KeyError:
            pass
        else:
            self.fail("KeyError was not raised when deleting invalid key %r "
                      "in resource properties %r" %
                      (invalid_key, org_init_props))

    def test_clear(self):
        """Test clearing the properties in a resource object."""
        init_props = {
            'prop1': 'abc',
            'prop2': 100042
        }
        res = MyResource(self.mgr, self.uri, init_props)

        res.properties.clear()

        self.assertEqual(len(res.properties), 0)


if __name__ == '__main__':
    unittest.main()



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
A **Logical Partition (LPAR)** is a subset of a physical z Systems or LinuxONE
computer that is not in DPM mode (Dynamic Partition Manager mode).
Objects of this class are not provided when the CPC is in DPM mode.

An LPAR is always contained in a CPC.

LPARs cannot be created or deleted by the user; they can only be listed.
"""

from __future__ import absolute_import

from ._manager import BaseManager
from ._resource import BaseResource
from ._logging import _log_call

__all__ = ['LparManager', 'Lpar']


class LparManager(BaseManager):
    """
    Manager object for LPARs. This manager object is scoped to the LPARs of a
    particular CPC.

    Derived from :class:`~zhmcclient.BaseManager`; see there for common methods
    and attributes.
    """

    def __init__(self, cpc):
        """
        Parameters:

          cpc (:class:`~zhmcclient.Cpc`):
            CPC defining the scope for this manager object.
        """
        super(LparManager, self).__init__(cpc)

    @property
    def cpc(self):
        """
        :class:`~zhmcclient.Cpc`: Parent object (CPC) defining the scope for
        this manager object.
        """
        return self._parent

    @_log_call
    def list(self, full_properties=False):
        """
        List the LPARs in scope of this manager object.

        Parameters:

          full_properties (bool):
            Controls whether the full set of resource properties should be
            retrieved, vs. only the short set as returned by the list
            operation.

        Returns:

          : A list of :class:`~zhmcclient.Lpar` objects.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        cpc_uri = self.cpc.get_property('object-uri')
        lpars_res = self.session.get(cpc_uri + '/logical-partitions')
        lpar_list = []
        if lpars_res:
            lpar_items = lpars_res['logical-partitions']
            for lpar_props in lpar_items:
                lpar = Lpar(self, lpar_props['object-uri'], lpar_props)
                if full_properties:
                    lpar.pull_full_properties()
                lpar_list.append(lpar)
        return lpar_list


class Lpar(BaseResource):
    """
    Representation of an LPAR.

    Derived from :class:`~zhmcclient.BaseResource`; see there for common
    methods and attributes.
    """

    def __init__(self, manager, uri, properties):
        """
        Parameters:

          manager (:class:`~zhmcclient.LparManager`):
            Manager object for this resource.

          uri (string):
            Canonical URI path of the LPAR object.

          properties (dict):
            Properties to be set for this resource object.
            See initialization of :class:`~zhmcclient.BaseResource` for
            details.
        """
        assert isinstance(manager, LparManager)
        super(Lpar, self).__init__(manager, uri, properties)

    @_log_call
    def activate(self, wait_for_completion=True):
        """
        Activate (start) this LPAR, using the HMC operation "Activate Logical
        Partition".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        lpar_uri = self.get_property('object-uri')
        body = {}
        result = self.manager.session.post(
            lpar_uri + '/operations/activate', body,
            wait_for_completion=wait_for_completion)
        return result

    @_log_call
    def deactivate(self, wait_for_completion=True):
        """
        De-activate (stop) this LPAR, using the HMC operation "Deactivate
        Logical Partition".

        Parameters:

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        lpar_uri = self.get_property('object-uri')
        body = {'force': True}
        result = self.manager.session.post(
            lpar_uri + '/operations/deactivate', body,
            wait_for_completion=wait_for_completion)
        return result

    @_log_call
    def load(self, load_address, wait_for_completion=True):
        """
        Load (boot) this LPAR from a load address (boot device), using the HMC
        operation "Load Logical Partition".

        Parameters:

          load_address (:term:`string`): Device number of the boot device.

          wait_for_completion (bool):
            Boolean controlling whether this method should wait for completion
            of the requested asynchronous HMC operation, as follows:

            * If `True`, this method will wait for completion of the
              asynchronous job performing the operation.

            * If `False`, this method will return immediately once the HMC has
              accepted the request to perform the operation.

        Returns:

          :term:`json object`:

            If `wait_for_completion` is `True`, returns None.

            If `wait_for_completion` is `False`, returns a JSON object with a
            member named ``job-uri``. The value of ``job-uri`` identifies the
            job that was started, and can be used with the
            :meth:`~zhmcclient.Session.query_job_status` method to determine
            the status of the job and the result of the asynchronous HMC
            operation, once the job has completed.

        Raises:

          :exc:`~zhmcclient.HTTPError`
          :exc:`~zhmcclient.ParseError`
          :exc:`~zhmcclient.AuthError`
          :exc:`~zhmcclient.ConnectionError`
        """
        lpar_uri = self.get_property('object-uri')
        body = {'load-address': load_address}
        result = self.manager.session.post(
            lpar_uri + '/operations/load', body,
            wait_for_completion=wait_for_completion)
        return result



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Example 4: Using the asynchronous interface.
"""

import sys
import logging
import yaml
import requests.packages.urllib3
import time

import zhmcclient

requests.packages.urllib3.disable_warnings()

if len(sys.argv) != 2:
    print("Usage: %s hmccreds.yaml" % sys.argv[0])
    sys.exit(2)
hmccreds_file = sys.argv[1]

with open(hmccreds_file, 'r') as fp:
    hmccreds = yaml.load(fp)

examples = hmccreds.get("examples", None)
if examples is None:
    print("examples not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

example4 = examples.get("example4", None)
if example4 is None:
    print("example4 not found in credentials file %s" % \
          (hmccreds_file))
    sys.exit(1)

loglevel = example4.get("loglevel", None)
if loglevel is not None:
    level = getattr(logging, loglevel.upper(), None)
    if level is None:
        print("Invalid value for loglevel in credentials file %s: %s" % \
              (hmccreds_file, loglevel))
        sys.exit(1)
    logging.basicConfig(level=level)

hmc = example4["hmc"]
cpcname = example4["cpcname"]
cpcstatus = example4["cpcstatus"]
lparname = example4["lparname"]

cred = hmccreds.get(hmc, None)
if cred is None:
    print("Credentials for HMC %s not found in credentials file %s" % \
          (hmc, hmccreds_file))
    sys.exit(1)

userid = cred['userid']
password = cred['password']

print(__doc__)

try:
    print("Using HMC %s with userid %s ..." % (hmc, userid))
    session = zhmcclient.Session(hmc, userid, password)
    cl = zhmcclient.Client(session)

    timestats = example4.get("timestats", None)
    if timestats:
        session.time_stats_keeper.enable()

    cpc = cl.cpcs.find(name=cpcname, status=cpcstatus)
    print("Status of CPC %s: %s" % \
          (cpc.properties['name'], cpc.properties['status']))

    lpar = cpc.lpars.find(name=lparname)
    print("Status of LPAR %s: %s" % \
          (lpar.properties['name'], lpar.properties['status']))

    print("De-Activating LPAR %s ..." % lpar.properties['name'])
    status = lpar.deactivate(wait_for_completion=False)

    print("job-uri: %s" % (status['job-uri']))
    job = session.query_job_status(status['job-uri'])
    print("job response: %s" % job)

    while job['status'] != 'complete':
        print("Job Status: %s" % (job['status']))
        time.sleep(1)
        job = session.query_job_status(status['job-uri'])

    print("job response: %s" % job)
    print('De-Activate complete !')

    print('Deleting completed job status ...')
    session.delete_completed_job_status(status['job-uri'])

#    print('Deleting completed job status again ...')
#    session.delete_completed_job_status(status['job-uri'])
#    Returns exception:
#    HTTPError: 404,1: No job or status for 'b571dbde-c9cb-11e1-8327-00215e676926_45fdd752-65f4-11e6-a6c3-00215e676926

    print("Logging off ...")
    session.logoff()

    if timestats:
        print(session.time_stats_keeper)

    print("Done.")

except zhmcclient.Error as exc:
    print("%s: %s" % (exc.__class__.__name__, exc))
    sys.exit(1)



#!/usr/bin/env python
# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Unit tests for _logging module.
"""

import logging
import unittest
from testfixtures import log_capture

from zhmcclient._logging import _log_call


class TestLogging(unittest.TestCase):
    """All test cases for the _log_call decorator."""

    @log_capture(level=logging.DEBUG)
    def test_logging_decorator(self, capture):
        """Simple test for the _log_call decorator."""

        @_log_call
        def do_something():
            """A decorated function that is called."""
            pass

        do_something()
        capture.check(('tests.test_logging',
                       'DEBUG',
                       'Entering test_logging_decorator.do_something()'),
                      ('tests.test_logging',
                       'DEBUG',
                       'Leaving test_logging_decorator.do_something()'))

# TODO: Add test cases for _get_logger(), specifically for null-handler



# Copyright 2016 IBM Corp. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This library logs calls to its public API and to some internal functions, but
doesn't output the log records by default.

It uses one :class:`~py:logging.Logger` object for each module. The name of
each such logger is the dotted module name (e.g. ``'zhmcclient._cpc'``). You
can also use the logger for the package name (``'zhmcclient'``) in order to
set up package-wide logging.

These loggers have a null-handler (see :class:`~py:logging.NullHandler`)
and have no log formatter (see :class:`~py:logging.Formatter`).

If you want to get the log output, add a log handler and set the log level
(see the :meth:`~py:logging.Logger.addHandler` and
:meth:`~py:logging.Logger.setLevel` methods of the
:class:`~py:logging.Logger` class).

For example, to output all warning level (and higher level) log records issued
by this library and by others to ``stdout`` in a particular format, do this:

::

    import logging

    handler = logging.StreamHandler()
    format_string = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    handler.setFormatter(logging.Formatter(format_string))
    logger = getLogger('')  # root logger
    logger.addHandler(handler)
    logger.setLevel(logging.WARNING)

If you want to see only the info level (and higher level) log records issued by
the ``_session`` module of this library, use the logger for just that module:

::

    import logging

    handler = logging.StreamHandler()
    format_string = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    handler.setFormatter(logging.Formatter(format_string))
    logger = getLogger('zhmcclient._session')
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
"""

import functools
import logging
import inspect


def _get_logger(name):
    """
    Return a :class:`~py:logging.Logger` object with the specified name.

    A :class:`~py:logging.NullHandler` is added to the logger (if it does not
    have one yet) in order for this library to be silent by default, and to
    leave the definition of 'real' logging handlers to the user of this
    library.
    """
    logger = logging.getLogger(name)
    if not any([isinstance(h, logging.NullHandler) for h in logger.handlers]):
        logger.addHandler(logging.NullHandler())
    # NOTE(markus_z): In case you want to test it locally, just replace the
    # NullHandler with the StreamHandler and set the loglevel to INFO.
    return logger


def _log_call(func):
    """
    Function decorator that causes the decorated function to log calls to
    itself to a logger.

    The logger's name is the dotted module name of the module defining the
    decorated function (e.g. 'zhmcclient._cpc').
    """
    mod = inspect.getmodule(func)
    try:
        modname = mod.__name__
    except AttributeError:
        raise TypeError("The _log_call decorator must be used on a function "
                        "(and not on top of the property decorator)")
    logger = _get_logger(modname)
    if inspect.isfunction(func):
        frames = inspect.getouterframes(inspect.currentframe())
        callername = frames[1][3]
        # At the time the decorator code gets control, the module has been
        # loaded, but class definitions of decorated methods are not complete
        # yet. Also, methods of the class are still functions at this point.
        if callername == '<module>':
            where = '{func}()'.format(func=func.__name__)
        else:  # it is a class name or outer function name
            where = '{caller}.{func}()'.format(caller=callername,
                                               func=func.__name__)
    else:
        raise TypeError("The _log_call decorator must be used on a function")

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        """
        Log entry to and exit from the decorated function, at the debug level.

        Note that this wrapper function is called every time the decorated
        function/method is called, but that the log message only needs to be
        constructed when logging for this logger and for this log level is
        turned on. Therefore, we do as much as possible in the decorator
        function, plus we use %-formatting and lazy interpolation provided by
        the log functions, in order to save resources in this function here.
        """
        logger.debug("Entering %s", where)
        result = func(*args, **kwargs)
        logger.debug("Leaving %s", where)
        return result

    return wrapper
